{-# OPTIONS_HADDOCK hide #-}

-- | Re-export event callbacks.
module Graphics.Gloss.Internals.Interface.Callback
        ( Callback(..)
        , DisplayCallback
        , KeyboardMouseCallback
        , MotionCallback
        , IdleCallback
        , ReshapeCallback)
where
import Graphics.Gloss.Internals.Interface.Backend.Types
